package com.sg.answer.sg;

import com.alibaba.fastjson.JSONObject;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.sg.answer.core.SgEntity;
import com.sg.answer.core.UserInfoEntity;
import com.sg.answer.entity.JobTypeEntity;
import com.sg.answer.panel.PlaceComponents;
import com.sg.answer.service.AnswerService;
import com.sg.answer.service.UserInfoService;
import com.sg.answer.utils.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.CookieStore;

import javax.swing.*;
import java.text.DecimalFormat;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicBoolean;

import static com.sg.answer.sg.API.*;
import static com.sg.answer.sg.Constant.*;
import static com.sg.answer.sg.Constant.gkNum;
import static com.sg.answer.sg.Constant.answerNum;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className FrameManager
 * @description TODO
 * @date 2020/3/19 13:14
 */
@Slf4j
public class FrameManager {

    private static final int MAX_LEVEL = 10;
    private static final DecimalFormat SCORE_FORMAT = new DecimalFormat("######0.00");
    private static final Set<String> processedJobs = ConcurrentHashMap.newKeySet();
    private static final ThreadPoolExecutor WORKER_POOL = new ThreadPoolExecutor(
            10, 20, 30L, TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(100),
            new ThreadFactoryBuilder().setNameFormat("AnswerWorker-%d").build(),
            new ThreadPoolExecutor.CallerRunsPolicy());

    public JPanel panel = new JPanel();
    private CookieStore cookie;

    public void init() {
        // 创建 JFrame 实例
        JFrame frame = new JFrame("全自动答题系统6.0.3 By 彭宇");
        frame.setBounds(500, 250, 770, 550);
        frame.setSize(770, 550);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        //初始化界面
        PlaceComponents pl = new PlaceComponents();
        panel = pl.placeComponents();
        frame.add(panel);
        // 设置界面可见
        frame.setVisible(true);
    }

    /**
     * @param loginButton,   userText, passwordText, comboBox, typeBox]
     * @param userInfoEntity
     * @param statrScore
     * @param status
     * @param pause
     * @param lock
     * @return void
     * @author wangyupeng
     * @description 登陆并开始刷题
     * @date 2019/7/5 15:30
     */
    public void start(JButton loginButton, SgEntity sgEntity, UserInfoEntity userInfoEntity, JLabel statrScore, AtomicBoolean status, AtomicBoolean pause, Object lock) throws Exception {
        if (!validateCredentials(sgEntity)) {
            return;
        }
        initializeSession(sgEntity);
        updateUIComponents(loginButton, false, "努力答题中……");

        log.info("当前活动线程数量:{}", Thread.activeCount());

        WORKER_POOL.execute(() -> {
            try {
                if (status.get()) {
                    cookie = login(sgEntity.getUserText().getText(), String.valueOf(sgEntity.getPasswordText().getPassword()));
                    UserInfoService.refreshUserInfo(cookie, sgEntity, userInfoEntity);
                    statrScore.setText(userInfoEntity.getIntegralDate().getText());
                    anwser(cookie, sgEntity, userInfoEntity, statrScore, status, pause, lock);
                } else {
                    log.info("答题已结束");
                }
            } catch (SgException sg) {
                log.error("答题结束:", sg);
                JOptionPane.showMessageDialog(panel, sg.getMessage(), " stop", JOptionPane.ERROR_MESSAGE);
                reset(loginButton, sgEntity, sg.getMessage());
            } catch (Exception e) {
                log.error("未知异常:", e);
            }
        });
    }

    public static void anwser(CookieStore cookie, SgEntity sgEntity, UserInfoEntity userInfoEntity, JLabel statrScore, AtomicBoolean status, AtomicBoolean pause, Object lock) throws Exception {
        //当前工种选项
        JobTypeEntity currentJobType = (JobTypeEntity) sgEntity.getComboBox().getSelectedItem();
        if (!status.get()) {
            return;
        }
        //获取当前用户答题进度关卡
        int gs = UserInfoService.getGKInfo(cookie, sgEntity);
        AnswerService service = new AnswerService(cookie);
        //外层循环控制1到10关，内存循环控制对应关数多少题数
        while (status.get()) {
            if (!status.get()) {
                log.info("结束本次刷题");
                break;
            }
            updateUIComponents(sgEntity.getGkText(), false, String.format("第%d关(%s)", gs, sgEntity.getDifficulty()));
            updateUIComponents(sgEntity.getTextArea(), false, "开始第" + gs + "关答题:");

            //获取用户历史答题关卡id
            int levelId = calculateLevelId(gs, sgEntity.getGz());

            log.info("当前工种：{}，开始第{}关答题,答题页面id：{}", currentJobType.getJobTypeNname(), gs, levelId);
            processedJobs.add(currentJobType.getJobTypeNname().trim());

            //打开当前闯关页面（开始答题计时）*/
            service.openAnswerHtml(levelId);

            //开始进行本官答题
            processLevels(sgEntity, status, pause, lock, gs, currentJobType, service, levelId);

            log.info(String.format("%s 开始第%s关结算，当前答题页面id：%d，闯关页面id:%d",currentJobType.getJobTypeNname(), gkNum.get(gs), levelId, sgEntity.getId()));
            log.info("当前答题工种数量：{},当前答题工种为：{}：", processedJobs.size(), StringUtils.join(processedJobs.toArray(), ","));
            if (!status.get()) {
                log.info("结束本次刷题,停止当前结算");
                break;
            }
            //更新UI答题进度
            sgEntity.apppendLogs("开始第" + gkNum.get(gs) + "关结算");
            //对当前关卡进行结算
            service.gkSettlement(levelId);

            //结算完成刷新个人信息答题成绩
            UserInfoService.refreshUserInfo(cookie, sgEntity, userInfoEntity);

            // 检查分数限制
            checkScoreLimit(userInfoEntity, sgEntity, statrScore, status);

            // 更新关卡计数
            if (gs >= MAX_LEVEL) {
                gs = 0;
            }
            gs++;
        }
    }

    private static void processLevels(SgEntity sgEntity, AtomicBoolean status, AtomicBoolean pause, Object lock, int gs, JobTypeEntity currentJobType, AnswerService service, int levelId) throws Exception {
        for (int j = 1; j <= answerNum.get(gs) + 10; j++) {
            if (!status.get()) {
                log.info("已结束本次刷题");
                break;
            }
            while (pause.get()) {
                sgEntity.apppendLogs(" 已暂停当前答题 ");
                onPause(lock);
            }

            log.info("{}====================>第{}题", currentJobType.getJobTypeNname(), j);
            sgEntity.apppendLogs(currentJobType.getJobTypeNname() + ":第" + j + "题,");

            JSONObject resJson = service.autoAnswer(stemList, levelId);

            if (Integer.parseInt(resJson.get("isright").toString()) == -1) {
                sgEntity.getTextArea().append("答题速度过快，直接进行下一题答题");
                continue;
            }
            if (Integer.parseInt(resJson.get("right_num").toString()) >= answerNum.get(gs)) {
                log.info("当前关已答题{}道", Integer.valueOf(resJson.get("right_num").toString()));
                break;
            }
            Thread.sleep(2000);
        }
    }


    private static void onPause(Object lock) {
        synchronized (lock) {
            try {
                log.info("已暂停答题，等待线程唤醒");
                lock.wait();
            } catch (InterruptedException e) {
               log.error("暂停答题异常", e);
            }
        }
    }

    // 登录严重
    private boolean validateCredentials(SgEntity sgEntity) throws SgException {
        String uname = Optional.ofNullable(sgEntity.getUserText())
                .map(JTextField::getText)
                .orElse("");
        String pwd = Optional.ofNullable(sgEntity.getPasswordText())
                .map(p -> new String(p.getPassword()))
                .orElse("");
        if (uname.isEmpty() || pwd.isEmpty()) {
            SwingUtilities.invokeLater(() ->
                    JOptionPane.showMessageDialog(panel, "账号密码不能为空", "错误", JOptionPane.ERROR_MESSAGE));
            return false;
        }
        return true;
    }


    // 初始化管卡数据
    private void initializeSession(SgEntity sgEntity) {
        sgEntity.setFlag(true);
        sgEntity.getTextArea().setText("");
        //当前工种选项
        JobTypeEntity currentJobType = (JobTypeEntity) sgEntity.getComboBox().getSelectedItem();
        //获取当前当前选择工种第一关关卡id
        sgEntity.setGz(currentJobType.getGkId());

        //获取当前选择难度id
        sgEntity.setType(level.get(sgEntity.getTypeBox().getSelectedItem().toString()));
        sgEntity.setDifficulty(sgEntity.getTypeBox().getSelectedItem().toString());

        //获取当前工种id(为自动获取当前答题关卡进度做准备)
        sgEntity.setId(currentJobType.getTypeId());
        log.info("当前选择工种为：{},【{}】第一关id为:【{}】", currentJobType.getJobTypeNname(), sgEntity.getTypeBox().getSelectedItem().toString(), currentJobType.getGkId());

        //根据难度选择计算关卡id变化
        switch (sgEntity.getType()) {
            case 1:
                break;
            case 2:
                sgEntity.setGz(sgEntity.getGz() + 10);
                break;
            case 3:
                sgEntity.setGz(sgEntity.getGz() + 20);
                break;
            default:
                break;
        }
    }

    // UI组件更新方法
    private static void updateUIComponents(JComponent component, boolean enabled, String text) {
        SwingUtilities.invokeLater(() -> {
            if (component instanceof AbstractButton) {
                component.setEnabled(enabled);
            }
            if (component instanceof JLabel) {
                ((JLabel) component).setText(text);
            } else if (component instanceof JButton) {
                ((JButton) component).setText(text);
            }
        });
    }

    private static int calculateLevelId(int baseLevel, int difficulty) {
        return (baseLevel > 10) ?
                baseLevel + difficulty - 2 :
                baseLevel + difficulty;
    }


    // 检查分数限制
    private static void checkScoreLimit(UserInfoEntity userInfoEntity, SgEntity sgEntity, JLabel statrScore, AtomicBoolean status) throws SgException {
        Double xj = Double.parseDouble(userInfoEntity.getIntegralDate().getText()) - Double.parseDouble(statrScore.getText());
        log.info("增长分数:{}", xj);
        updateUIComponents(sgEntity.getXjText(), true, SCORE_FORMAT.format(xj));

        if (!ObjectUtils.isEmpty(sgEntity.getCoustomScore()) && xj >= sgEntity.getCoustomScore() && sgEntity.getCoustomScore() != 0.0) {
            status.set(false);
            log.info("已完成{}分刷分,如需继续请重新开始", sgEntity.getCoustomScore());
            throw new SgException(ErrorCode.SORCE_LIMT.getCode(), ErrorCode.SORCE_LIMT.getName());
        }
    }
}